
DETERMINE_NIVEAU_COLLECTEUR:
	;
	; dtermine si le niveau de tension correspond  un transistor satur (1)
	; ou bloqu (0)
	; entre: W = niveau de tension
	; sortie: Z=1 si niveau bas Z=0 si niveau haut
	;         C=0 si niveaux corrects C=1 si niveaux incorrects
	;
	movwf	FSR
	movlw	NIVEAU_MAX_SAT
	subwf	FSR,W			; C=0 si ok
	btfss	STATUS,C
	goto	low_collector
	movlw	NIVEAU_MIN_BLOC
	subwf	FSR,W
	btfsc	STATUS,C
	goto	high_collector
	bsf	STATUS,C
	return

low_collector:
	bsf	STATUS,Z
	bcf	STATUS,C
	return

high_collector:
	bcf	STATUS,Z
	bcf	STATUS,C
	return

READ_VOLTAGE_8:
	;
	; lecture d'une valeur sur 8 bits sur une des entres
	; entre: W = SIGNAL_X  X = LEFT, MIDDLE ou RIGHT (0, 1 ou 2)
	; retour: W = valeur du convertisseur
	; modif du 7/11/2003: on effectue une double conversion pour viter
	; une erreur de mesure due  l'chantillonnage
	
	movwf	FSR
	movlw	2
	movwf	first_conversion
loop_conversion:
	BANK1
	movlw	B'00000100'		; AN0 AN1 AN3 left justified
	movwf	ADCON1
	BANK0
	movf	FSR,f
	btfsc	STATUS,Z
	goto	read_voltage_left
	btfss	FSR,0
	goto	read_voltage_right
	;
	; lecture voltage du milieu
	;
	movlw	B'00001001'
	goto	start_conversion

read_voltage_right:
	;
	; lecture du voltage de droite
	;
	movlw	B'00000001'
	goto	start_conversion

read_voltage_left:	
	;
	; lecture du voltage de gauche
	;
	movlw	B'00011001'

start_conversion:
	movwf	ADCON0
	nop
	nop
	bsf	ADCON0,GO
wait_conversion:
	btfsc	ADCON0,NOT_DONE
	goto	wait_conversion
	decfsz	first_conversion,f
	goto	loop_conversion
	movf	ADRESH,W
	return

READ_VOLTAGE_10:
	;
	; lecture d'une valeur sur 10 bits sur une des entres
	; entre: W = SIGNAL_X  X = LEFT, MIDDLE ou RIGHT (0, 1 ou 2)
	; retour: W = 8 bits bas
	;         FSR = 2 bits haut
	;
	movwf	FSR
	BANK1
	movlw	B'10000100'		; AN0 AN1 AN3 right justified
	movwf	ADCON1
	BANK0	
	movf	FSR,f
	btfsc	STATUS,Z
	goto	read_voltage_left_10
	btfss	FSR,0
	goto	read_voltage_right_10
	;
	; lecture voltage du milieu
	;
	movlw	B'00001001'
	goto	start_conversion_10

read_voltage_right_10:
	;
	; lecture du voltage de droite
	;
	movlw	B'00000001'
	goto	start_conversion_10

read_voltage_left_10:	
	;
	; lecture du voltage de gauche
	;
	movlw	B'00011001'

start_conversion_10:
	movwf	FSR
	clrf	data_l
	clrf	data_h
	movlw	.16		; moyenne sur 16 valeurs
	movwf	cptr		; compteur de boucle
loop_conv_10:
	movf	FSR,W
	movwf	ADCON0
	nop
	nop
	bsf	ADCON0,GO
wait_conversion_10:
	btfsc	ADCON0,NOT_DONE
	goto	wait_conversion_10

	BANK1
	movf	ADRESL,W
	BANK0
	addwf	data_l,f
	btfsc	_C
	incf	data_h,f
	movf	ADRESH,W
	addwf	data_h,f
	decfsz	cptr,f
	goto	loop_conv_10
	bcf	_C
	rrf	data_h,f
	rrf	data_l,f
	bcf	_C
	rrf	data_h,f
	rrf	data_l,f
	bcf	_C
	rrf	data_h,f
	rrf	data_l,f
	bcf	_C
	rrf	data_h,f
	rrf	data_l,f
	movf	data_h,W	
	movwf	FSR
	movf	data_l,W
	return

INIT_ADC:
	;
	; initialisation du convertisseur A/D
	;
	BANK1
	movlw	B'00000100'
	movwf	ADCON1
	BANK0
	return

SET_RESISTANCE:
	;
	; SET_RESISTANCE: configure une resistance donnee sur une sortie donnee
	; entree: W High nibble: code de la resistance Low Nibble: code de position
	; code de la rsistance: 1  4 ; code de position : 0 (gauche)  2 (droite)
	movwf	FSR			; sauvegarde
	swapf	FSR,W
	andlw	0x07			; W = valeur de la rsistance
	;
	btfss	FSR,0			; skip si 1 (MIDDLE)
	goto	set_resistance_1
	;
	; rsistance du milieu
	;
	movwf	FSR			; sauvegarde temporaire de l'offset
	movlw	LOW TABLE_RESISTANCE_MILIEU
	addwf	FSR,f
	movlw	HIGH TABLE_RESISTANCE_MILIEU
	btfsc	STATUS,C
	addlw	1
	movwf	PCLATH
	movf	FSR,W
	call	TABLE_RESISTANCE_MILIEU
	movwf	FSR			; les bits 0 et correspondent aux entres des multiplexeurs
	bcf	RES_MIDDLE_L
	bcf	RES_MIDDLE_H
	btfsc	FSR,0
	bsf	RES_MIDDLE_L
	btfsc	FSR,1
	bsf	RES_MIDDLE_H
	return
	
set_resistance_1:
	btfss	FSR,1			; skip si 2 (RIGHT)
	goto	set_left_resistance
	;
	; rsistance de droite
	;
	movwf	FSR			; sauvegarde temporaire de l'offset
	movlw	LOW TABLE_RESISTANCE_DROITE
	addwf	FSR,f
	movlw	HIGH TABLE_RESISTANCE_DROITE
	btfsc	STATUS,C
	addlw	1
	movwf	PCLATH
	movf	FSR,W
	call	TABLE_RESISTANCE_DROITE
	movwf	FSR			; les bits 0 et correspondent aux entres des multiplexeurs
	bcf	RES_RIGHT_L
	bcf	RES_RIGHT_H
	btfsc	FSR,0
	bsf	RES_RIGHT_L
	btfsc	FSR,1
	bsf	RES_RIGHT_H
	return
	
set_left_resistance:
	;
	; rsistance de gauche
	;
	movwf	FSR			; sauvegarde temporaire de l'offset
	movlw	LOW TABLE_RESISTANCE_GAUCHE
	addwf	FSR,f
	movlw	HIGH TABLE_RESISTANCE_GAUCHE
	btfsc	STATUS,C
	addlw	1
	movwf	PCLATH
	movf	FSR,W
	call	TABLE_RESISTANCE_GAUCHE
	movwf	FSR			; les bits 0 et correspondent aux entres des multiplexeurs
	bcf	RES_LEFT_L
	bcf	RES_LEFT_H
	btfsc	FSR,0
	bsf	RES_LEFT_L
	btfsc	FSR,1
	bsf	RES_LEFT_H
	return

TABLE_RESISTANCE_DROITE:
	movwf	PCL
	DT	0x03,0x02,0x01,0x00	; 0R,1K,5K,100K

TABLE_RESISTANCE_MILIEU
	movwf	PCL
	DT	0x03,0x02,0x01,0x00	; 0R,1K,5K,100K

TABLE_RESISTANCE_GAUCHE
	movwf	PCL
	DT	0x03,0x02,0x01,0x00	; 0R,1K,5K,100K


	; SET_VOLTAGE: configure la tension (0 ou 5V) sur une sortie donnee
	; entree: W High nibble: code du voltage Low Nibble: code de position
	;
SET_VOLTAGE:
	movwf	temporaire		; sauvegarde
					; position 0, 1 ou 2 (GAUCHE, MIDDLE, RIGHT)
	btfss	temporaire,0		; skip si 1 (MIDDLE)
	goto	set_voltage_1
	; voltage du milieu
	btfss	temporaire,4
	goto	set_middle_voltage_0V
	bsf	OUTPUT_MIDDLE
	return
set_middle_voltage_0V:
	bcf	OUTPUT_MIDDLE
	return
	
set_voltage_1:
	btfss	temporaire,1		; skip si 2 (RIGHT)
	goto	set_left_voltage
	; voltage de droite
	btfss	temporaire,4
	goto	set_right_voltage_0V
	bsf	OUTPUT_RIGHT
	return
set_right_voltage_0V:
	bcf	OUTPUT_RIGHT
	return
	
set_left_voltage:
	; voltage de gauche
	btfss	temporaire,4
	goto	set_left_voltage_0V
	bsf	OUTPUT_LEFT
	return
set_left_voltage_0V:
	bcf	OUTPUT_LEFT
	return


DELAY_CONVERSION:
	call	LCD_SHORTDELAY
	call	LCD_SHORTDELAY
	call	LCD_SHORTDELAY
	call	LCD_SHORTDELAY
	call	LCD_SHORTDELAY
	return
