; Programme principal du testeur de transistor
; Version: 1.0
; Date: 7/10/2003
; 
; By M. Waleczek
;
;; PIC setup and toplevel includes

;	LIST	P=16F877
;	INCLUDE "P16F877.inc"
	LIST	P=16F876
	INCLUDE "P16F876.inc"
	INCLUDE	"banks.inc"

; Pindefs



#DEFINE	LCD_D7	PORTC,0
#DEFINE	LCD_D6	PORTC,1
#DEFINE	LCD_D5	PORTC,2
#DEFINE	LCD_D4	PORTC,3
#DEFINE	LCD_E	PORTA,5
#DEFINE	LCD_RS	PORTC,7

; pindefs commande des resistances

#DEFINE RES_LEFT_L	PORTB,1
#DEFINE RES_LEFT_H	PORTB,0

#DEFINE RES_MIDDLE_L 	PORTA,2
#DEFINE RES_MIDDLE_H 	PORTB,2

#DEFINE RES_RIGHT_L	PORTB,4
#DEFINE RES_RIGHT_H 	PORTB,5

; Pindefs des sorties

#DEFINE OUTPUT_LEFT	PORTC,5
#DEFINE OUTPUT_MIDDLE	PORTC,4
#DEFINE OUTPUT_RIGHT	PORTC,6

#DEFINE TYPE_NPN	0x01
#DEFINE TYPE_PNP	0x02
#DEFINE TYPE_INCONNU	0x03
#DEFINE TYPE_NJFET	0x04
#DEFINE TYPE_PJFET	0x05
#DEFINE TYPE_NMOSFET	0x06
#DEFINE TYPE_PMOSFET	0x07

#DEFINE RESISTANCE_100R	0x10
#DEFINE RESISTANCE_1K	0x20
#DEFINE RESISTANCE_5K	0x30
#DEFINE RESISTANCE_100K	0x40

#DEFINE R100R		.100
#DEFINE RPICLOW		.25		; rsistance interne du pic pour niveau bas
#DEFINE RPICHIGH	.64		; rsistance interne du pic pour niveau haut
#DEFINE RSDEFAUT	.130		; rsistance des commutateurs analogiques par dfaut


#DEFINE TYPE_EMETTEUR	'E'
#DEFINE TYPE_BASE	'B'
#DEFINE TYPE_COLLECTEUR 'C'
#DEFINE TYPE_SOURCE	'S'
#DEFINE TYPE_GRILLE	'G'
#DEFINE TYPE_DRAIN	'D'

#DEFINE SIGNAL_LEFT	0x00
#DEFINE SIGNAL_MIDDLE	0x01
#DEFINE SIGNAL_RIGHT	0x02

#DEFINE BASE_LEFT	0x00
#DEFINE BASE_MIDDLE	0x10
#DEFINE BASE_RIGHT	0x20

#DEFINE POSITION_LEFT	0x00
#DEFINE POSITION_MIDDLE	0x01
#DEFINE POSITION_RIGHT	0x02

#DEFINE VOLTAGE_0V	0x00
#DEFINE VOLTAGE_5V	0x10

#DEFINE MAXIMUM_LOW_LEVEL_BIP	.50
#DEFINE MINIMUM_LOW_LEVEL_BIP	.25
#DEFINE MINIMUM_HIGH_LEVEL_BIP	.230
#DEFINE LIMITE_FUITE_COLLECTEUR	.50

#DEFINE MAXIMUM_LOW_LEVEL_JFET	.25
#DEFINE MINIMUM_LOW_LEVEL_JFET	.5
#DEFINE MINIMUM_HIGH_LEVEL_PJFET .250	; 4,9V
#DEFINE MINIMUM_HIGH_LEVEL_NJFET .30	; 0,58V
#DEFINE MAXIMUM_HIGH_LEVEL_PJFET .255	; 5V
#DEFINE MAXIMUM_HIGH_LEVEL_NJFET .50	; 1V

#DEFINE COLLECTEUR_GAUCHE	0x00
#DEFINE COLLECTEUR_MILIEU	0x10
#DEFINE COLLECTEUR_DROITE	0x20

#DEFINE EMETTEUR_GAUCHE		0x00
#DEFINE EMETTEUR_MILIEU		0x01
#DEFINE EMETTEUR_DROITE		0x02

#DEFINE R_G_M		0
#DEFINE R_G_D		1
#DEFINE R_M_D		2
#DEFINE CODE_EEPROM	0X55
#DEFINE ADR_CODE_EEPROM	3

; niveaux de dtection saturation/blocage du transistor

#DEFINE NIVEAU_MAX_SAT		.25
#DEFINE NIVEAU_MIN_BLOC		.100

#DEFINE FREQUENCE_CLOCK		.1					; frquence du pic en MHz 
#DEFINE DUREE_TIMER_1		.50					; priode de l'interruption TIMER1 en ms
#DEFINE RECHARGE_TIMER1		DUREE_TIMER_1*.250*FREQUENCE_CLOCK
#DEFINE DUREE_VERSION		.2					; dure affichage version en s
#DEFINE DUREE_AFF_RESISTANCE 	.50

#DEFINE CODE_MESSAGE_INVITE	1
#DEFINE CODE_MESSAGE_CAL	2
#DEFINE CODE_MESSAGE_ERRCAL	3

#DEFINE	_C	STATUS,0
#DEFINE	_Z	STATUS,2

SIGN equ     0x15    ; save location for sign in MSB
TEMPB3	equ 0x1C
FPFLAGS	equ 0x16    ; floating point library exception flags
NAN	equ	4	; bit4 = not-a-number exception flag

B0	equ	0
B1	equ	1
B2	equ	2
B3	equ	3
B4	equ	4
B5	equ	5
B6	equ	6
B7	equ	7

MSB	equ	7
LSB	equ	0

CONFIG_RESISTANCE MACRO rleft,rmiddle,rright

	movlw	SIGNAL_LEFT|rleft
	call	SET_RESISTANCE
	movlw	SIGNAL_MIDDLE|rmiddle
	call	SET_RESISTANCE
	movlw	SIGNAL_RIGHT|rright
	call	SET_RESISTANCE

	ENDM

CONFIG_VOLTAGE MACRO vleft, vmiddle, vright

	movlw	SIGNAL_LEFT|vleft
	call	SET_VOLTAGE
	movlw	SIGNAL_MIDDLE|vmiddle
	call	SET_VOLTAGE
	movlw	SIGNAL_RIGHT|vright
	call	SET_VOLTAGE

	ENDM

CONFIG_BROCHAGE MACRO left, middle, right

	movlw	SIGNAL_LEFT		; test avec base  gauche
	movwf	left
	if left==position_emetteur
	  movlw TYPE_EMETTEUR
	endif
	if left==position_collecteur
	  movlw	TYPE_COLLECTEUR
	endif
	if left==position_base
	  movlw	TYPE_BASE
	endif
	movwf	pin_left

	movlw	SIGNAL_MIDDLE
	movwf	middle
	if middle==position_emetteur
	  movlw TYPE_EMETTEUR
	endif
	if middle==position_collecteur
	  movlw	TYPE_COLLECTEUR
	endif
	if middle==position_base
	  movlw	TYPE_BASE
	endif
	movwf	pin_middle

	movlw	SIGNAL_RIGHT
	movwf	right
	if right==position_emetteur
	  movlw TYPE_EMETTEUR
	endif
	if right==position_collecteur
	  movlw	TYPE_COLLECTEUR
	endif
	if right==position_base
	  movlw	TYPE_BASE
	endif
	movwf	pin_right

	ENDM

	__badram	H'70',H'BB'-H'BF'

; Variables
	CBLOCK	0x20
		temporaire
		pin_left
		pin_middle
		pin_right
		data_h
		data_l
		decimal_1
		decimal_10
		decimal_100
		masque
		cptr
		save_status
		save_w
		tempo1
		voltage_left
		voltage_middle
		voltage_right		; 30h
		terminal_levels
		transistor_type
		previous_transistor_type
		collecteur_1
		emetteur_1
		voltage_collecteur_1
		niveau_logique_1
		collecteur_2
		emetteur_2
		voltage_collecteur_2
		niveau_logique_2
		polarisation_base
		polarisation_emetteur
		polarisation_collecteur
		position_base
		position_collecteur	; 40h
		position_emetteur
		voltage_emetteur
		voltage_base
		courant_base
		vb1
		vb2
		vb3
		vb4
		jfet_count_low
		jfet_count_high_n
		jfet_count_high_p
		voltage_drain_h
		voltage_drain_l
		voltage_source_h
		voltage_source_l	; 50h
		flag_transistor_inconnu
		resistance_gauche_milieu
		resistance_gauche_droite
		resistance_milieu_droite	; rsistance des commutateurs
		sauvegarde_effectuee		; code pour sauvegarde eeprom
		resistance_serie		; resistance des commutateurs pour mesure Rdson
		polarisation_base_zero		; pour calcul du beta
		voltage_emetteur_zero

		; pour LCD

		__LCD_DOCBUFF		; Display Control settings

		; data pour conversion A/D

		first_conversion
	ENDC

	CBLOCK	0x71
		REMB0
		REMB1
		AARGB0
		AARGB1
		AARGB2
		AARGB5
		BARGB0
		BARGB1
		TEMP
		TEMPB0
		TEMPB1
		LOOPCOUNT
		
	ENDC

; Reset vector
	ORG	0
	nop				; pour emulation
	goto	START
	
	; Interrupt vector
	
	ORG	4
	movwf	save_w			; sauvegarde W
	swapf	STATUS,w
	movwf	save_status		; sauvegarde status swap
	BANK0
	btfsc	PIR1,TMR1IF		; overflow timer 1
	goto	TIMER1_INTERRUPT

END_OF_INTERRUPT:
	swapf	save_status,W
	movwf	STATUS
	swapf	save_w,f
	swapf	save_w,W
	retfie

TIMER1_INTERRUPT:
	
	; interruption du timer 1 toutes les 50 ms pour tempos longues
	
	bcf	PIR1,TMR1IF		; clear flag
	movf	tempo1,f
	btfss	_Z
	decf	tempo1,f		; dcrmente tempo1 si pas  0
	movlw	LOW (0xFFFF-RECHARGE_TIMER1)
	movwf	TMR1L
	movlw	HIGH (0xFFFF-RECHARGE_TIMER1)
	movwf	TMR1H
	goto	END_OF_INTERRUPT

	
	; Start of code
	
START:
	; init i/o pour afficheur lcd
	
	BANK1
	movlw	0x00
	movwf	TRISC			; bits du PORT C en sorties
	BANK0
	clrf	PORTC
	BANK1
	movlw	B'11011011'		; RA5 sortie pour RS afficheur
	movwf	TRISA

	movlw	B'11001000'
	movwf	TRISB
	BANK0
	call	INIT_TIMER1
	call	INIT_ADC
	call	CLEAR_SIGNAL
	call	LCD_INIT		; Init the LCD
	call	LCD_CLRSCR
	movlw	CODE_MESSAGE_INVITE
	call	DISPLAY_MESSAGE		; affichage de l'invite et numro de version
	movlw	DUREE_VERSION*.1000/DUREE_TIMER_1
	call	TEMPORISATION
	call	TEST_CALIBRATION	; retourne W=0 en mode normal W=1 en mode calibration
	addlw	0
	btfss	_Z			; skip si pas en mode calibration
	call	CALIBRATION

	; si la calibration n'a pas t effectue on l'indique

	call	TEST_CAL_DONE		; retourne Z=1 si cal effectue
	btfss	_Z			; skip si calibration effectue
	goto	main_01
	call	LECTURE_RESISTANCES	; rsistances srie en EEPROM
	goto	main_02
main_01:
	call	LCD_CLRSCR
	movlw	CODE_MESSAGE_ERRCAL
	call	DISPLAY_MESSAGE		; affiche message 'err cal'
	movlw	.40
	call	TEMPORISATION
	movlw	RSDEFAUT
	movwf	resistance_gauche_milieu
	movwf	resistance_gauche_droite
	movwf	resistance_milieu_droite ; init des valeurs par dfaut
main_02:
	call	LCD_CLRSCR
	movlw	TYPE_INCONNU
	movwf	previous_transistor_type
	bcf	flag_transistor_inconnu,0
	;call	DISPLAY_TYPE_TRANSISTOR	; initialisation affichage transistor inconnu
	
main_loop:
	call	MESURE_VOLTAGES		; mesure les tensions sur les 3 electrodes
	call	TEST_FOR_BIPOLAR	; retourne le type dans W
	movwf	FSR
	xorlw	TYPE_INCONNU
	btfsc	_Z
	goto	test_jfet
	bcf	flag_transistor_inconnu,0
	movf	FSR,W
	andlw	0x0F			; type de transistor dans nibble faible
	movwf	transistor_type
	swapf	FSR,W			; position de la base dans nibble fort
	andlw	0x0F			; 0:gauche, 1:milieu 2: droite
	movwf	FSR			; sauvegarde position de base
	
	; le transistor a la signature d'un bipolaire
	
	call	DETERMINE_SIGNAL_BASE	; dtermine le signal de base
	movf	FSR,W			; FSR = position de la base (0=gauche, ... )
	call	DETERMINE_BROCHAGE_BIPOLAIRE	; retourne W=pos. coll. (quartet haut) et pos. emett. (quartet faible)
	btfss	_C
	goto	main_brochage_bipolaire
	
	; il faut tester ici les mosfet et les bipolaires avec diode

main_032:
	movlw	TYPE_NPN
	movwf	transistor_type		; on essaie le type NPN
	call	TEST_BIPOLAR_WITH_DIODE	; retourne C=1 si erreur
	btfss	_C
	goto	main_033		; OK
	
	movlw	TYPE_PNP
	movwf	transistor_type		; on essaie le type NPN
	call	TEST_BIPOLAR_WITH_DIODE	; retourne C=1 si erreur
	btfsc	_C			; saut si PNP valide
	goto	main_03

main_033:
	
	; si le transistor est de type mosfet on va changer le nom des broches

	bcf	flag_transistor_inconnu,0
	movf	transistor_type,W
	xorlw	TYPE_NMOSFET
	btfsc	_Z
	goto	main_034		; jump si mosfet
	
	movf	transistor_type,W
	xorlw	TYPE_PMOSFET
	btfss	_Z
	goto	main_035		; jump si pas un mosfet

main_034:

	; transistor MOSFET: changement de noms (collecteur->drain, etc...)

	movf	pin_left,W
	call	BIPOLAR_TO_MOSFET
	movwf	pin_left

	movf	pin_right,W
	call	BIPOLAR_TO_MOSFET
	movwf	pin_right

	movf	pin_middle,W
	call	BIPOLAR_TO_MOSFET
	movwf	pin_middle

	movf	transistor_type,W
	call	DISPLAY_TYPE_TRANSISTOR
	call	DISPLAY_BROCHAGE_TRANSISTOR
	call	BINAIRE_TO_DECIMAL		; traduit data_h:data_l
	call	DISPLAY_VTH
	;call	TEMPO_MAIN_LOOP
	goto	main_loop

main_035:
	movf	transistor_type,W
	call	DISPLAY_TYPE_TRANSISTOR
	call	DISPLAY_BROCHAGE_TRANSISTOR
	goto	main_04

main_03:

	; transistor invalide

	call	AFFICHE_TRANSISTOR_INCONNU
	goto	main_loop

main_brochage_bipolaire:
	call	CLEAR_SIGNAL
	movf	transistor_type,W
	call	DISPLAY_TYPE_TRANSISTOR
	call	DISPLAY_BROCHAGE_TRANSISTOR
	
main_04:
	call	BINAIRE_TO_DECIMAL
	call	DISPLAY_BETA_TRANSISTOR	; affiche decimal_100:decimal_10:decimal_100
	;call	TEMPO_MAIN_LOOP
	goto	main_loop

test_jfet:
	
	; type non bipolaire; test pour JFET
	
	call	TEST_FOR_JFET		; retourne W=position grille:type transistor
	movwf	FSR
	xorlw	TYPE_INCONNU
	btfss	_Z			; skip si pas un JFET
	goto	main_brochage_jfet
	goto	main_032		; test bipolaires spciaux et mosfet

main_brochage_jfet:
	
	; le transistor a la signature d'un JFET
	
	bcf	flag_transistor_inconnu,0
	movf	FSR,W
	andlw	0x0F			; type de transistor dans nibble faible
	movwf	transistor_type
	swapf	FSR,W			; position de la base dans nibble fort
	andlw	0x0F			; 0:gauche, 1:milieu 2: droite
	movwf	FSR			; sauvegarde position de la grille
	
	movlw	'*'
	movwf	pin_right
	movwf	pin_middle
	movwf	pin_left
	movf    FSR,W				; FSR=position grille
	call	DETERMINE_SIGNAL_GRILLE		; dtermine le signal de grille (idem bipolaire)
	
	; la position de la grille est dtermine (contenue dans FSR)
	; reste a trouver le source et le drain
	
	movf	FSR,W
	call	DETERMINE_BROCHAGE_FET
	call	CLEAR_SIGNAL
	movf	transistor_type,W
	call	DISPLAY_TYPE_TRANSISTOR
	call	DISPLAY_BROCHAGE_TRANSISTOR	; calcul aussi le RDSON
	call	BINAIRE_TO_DECIMAL
	call	DISPLAY_RDSON
	call	TEMPO_MAIN_LOOP
	goto	main_loop

AFFICHE_TRANSISTOR_INCONNU:
	
	; affichage transistor inconnu
	
	btfsc	flag_transistor_inconnu,0	; pour n'afficher qu'une fois
	return

	bsf	flag_transistor_inconnu,0
	call	CLEAR_SIGNAL
	call	LCD_CLRSCR
	movlw	TYPE_INCONNU
	call	DISPLAY_TYPE_TRANSISTOR
	movlw	'*'
	movwf	pin_right
	movwf	pin_middle
	movwf	pin_left
	call	DISPLAY_BROCHAGE_TRANSISTOR
	;call	TEMPO_MAIN_LOOP
	return

TEMPO_MAIN_LOOP:
	
	; temporisation pour la boucle principale
	; vite les instabilits d'affichage et limite la consommation
	
	movlw	.5
	movwf	tempo1
loop_tempo:
	movf	tempo1,f
	btfss	_Z
	goto	loop_tempo
	return

TEST_FOR_BIPOLAR:
	;
	; un seul niveau bas -> transistor NPN
	; un seul niveau haut -> transistor PNP
	; la routine retourne le type de transistor dans le quartet faible de W
	; la position de la base est retourne dans le quartet faible de W
	
	; pin gauche
	
	clrf	terminal_levels
	movlw	MAXIMUM_LOW_LEVEL_BIP
	subwf	voltage_left,W
	btfss	_C				; skip si au-dessus du niveau bas
	goto	test_left_minimum_level		; en-dessous du niveau bas -> test pour minimum de tension

	movlw	MINIMUM_HIGH_LEVEL_BIP
	subwf	voltage_left,W
	btfss	_C				; skip si au-dessus du niveau haut
	goto	erreur_voltage_bipolaire	; entre niveau bas et niveau haut -> erreur

	bsf	terminal_levels,SIGNAL_LEFT
	goto	test_middle_terminal

test_left_minimum_level:

	movlw	MINIMUM_LOW_LEVEL_BIP
	subwf	voltage_left,W
	btfss	_C				; skip si au-dessus du niveau bas minimum
	goto	erreur_voltage_bipolaire	; pas un bipolaire
	
	; pin du milieu
	
test_middle_terminal:

	movlw	MAXIMUM_LOW_LEVEL_BIP
	subwf	voltage_middle,W
	btfss	_C				; skip si au-dessus du niveau bas
	goto	test_middle_minimum_level	; en-dessous du niveau bas -> test pour minimum de tension

	movlw	MINIMUM_HIGH_LEVEL_BIP
	subwf	voltage_middle,W
	btfss	_C				; skip si au-dessus du niveau haut
	goto	erreur_voltage_bipolaire 	; entre niveau bas et niveau haut -> erreur

	bsf	terminal_levels,SIGNAL_MIDDLE	
	goto	test_right_terminal

test_middle_minimum_level:
	movlw	MINIMUM_LOW_LEVEL_BIP
	subwf	voltage_middle,W
	btfss	_C				; skip si au-dessus du minimum
	goto	erreur_voltage_bipolaire 	; pas un bipolaire
	
	; pin de droite
	
test_right_terminal:

	movlw	MAXIMUM_LOW_LEVEL_BIP
	subwf	voltage_right,W
	btfss	_C			; skip si au-dessus du niveau bas
	goto	test_right_minimum_level ; en-dessous du niveau bas -> fin du traitement

	movlw	MINIMUM_HIGH_LEVEL_BIP
	subwf	voltage_right,W
	btfss	_C			; skip si au-dessus du niveau haut
	goto	erreur_voltage_bipolaire ; entre niveau bas et niveau haut -> erreur

	bsf	terminal_levels,SIGNAL_RIGHT
	goto	end_test_terminal

test_right_minimum_level:

	movlw	MINIMUM_LOW_LEVEL_BIP
	subwf	voltage_right,W
	btfss	_C			; skip si au-dessus du minimum
	goto	erreur_voltage_bipolaire

end_test_terminal:
	
	; les flags de terminal_levels sont positionns
	
	movlw	HIGH TABLE_BIPOLAIRE
;	movlw	TABLE_BIPOLAIRE
	movwf	PCLATH
	movf	terminal_levels,W
	call	TABLE_BIPOLAIRE
	return

erreur_voltage_bipolaire:
	
	; tension incorrecte sur une des broches du transistor
	
	movlw	TYPE_INCONNU
	return

TEST_BIPOLAR_WITH_DIODE:
	
	; test pour les transistors avec diode entre Col/Emet (genre BF198)
	
	CONFIG_BROCHAGE position_emetteur, position_base, position_collecteur
	call	CALCUL_BETA		; retourne C=0 si OK
	btfss	_C
	return

	CONFIG_BROCHAGE position_emetteur, position_collecteur, position_base
	call	CALCUL_BETA
	btfss	_C
	return

	CONFIG_BROCHAGE position_collecteur, position_base, position_emetteur
	call	CALCUL_BETA
	btfss	_C
	return

	CONFIG_BROCHAGE position_collecteur, position_emetteur, position_base
	call	CALCUL_BETA
	btfss	_C
	return

	CONFIG_BROCHAGE position_base, position_collecteur, position_emetteur
	call	CALCUL_BETA
	btfss	_C
	return

	CONFIG_BROCHAGE position_base, position_emetteur, position_collecteur
	call	CALCUL_BETA
	btfss	_C
	return

	; le transistor n'est pas un bipolaire

	return			; C = 1

MESURE_VOLTAGES:
	;
	; on positionne deux des broches  la masse (100 ohms) , la troisime au +5V
	;  travers une rsistance de 5K.
	; les tensions sont mmorises dans voltage_left, voltage_middle et voltage_right
	;
	
	; test en activant la broche gauche avec 5K, les 2 autres  la masse
	
	CONFIG_RESISTANCE RESISTANCE_5K, RESISTANCE_100R, RESISTANCE_100R
	
	CONFIG_VOLTAGE VOLTAGE_5V, VOLTAGE_0V, VOLTAGE_0V
	
	call	LCD_LONGDELAY

	; lecture de la tension
	
	movlw	SIGNAL_LEFT
	call	READ_VOLTAGE_8
	movwf	voltage_left
	movlw	SIGNAL_MIDDLE
	call	READ_VOLTAGE_8
	subwf	voltage_left,f
	
	; test en activant la broche du milieu avec 5K, les 2 autres  la masse
	
	CONFIG_RESISTANCE RESISTANCE_100R, RESISTANCE_5K, RESISTANCE_100R

	CONFIG_VOLTAGE VOLTAGE_0V, VOLTAGE_5V, VOLTAGE_0V

	call	LCD_LONGDELAY

	; lecture de la tension

	movlw	SIGNAL_MIDDLE
	call	READ_VOLTAGE_8
	movwf	voltage_middle
	movlw	SIGNAL_LEFT
	call	READ_VOLTAGE_8
	subwf	voltage_middle,f

	; test en activant la broche du milieu avec 5K, les 2 autres  la masse

	CONFIG_RESISTANCE RESISTANCE_100R, RESISTANCE_100R, RESISTANCE_5K

	CONFIG_VOLTAGE VOLTAGE_0V, VOLTAGE_0V, VOLTAGE_5V

	call	LCD_LONGDELAY

	; lecture de la tension

	movlw	SIGNAL_RIGHT
	call	READ_VOLTAGE_8
	movwf	voltage_right
	movlw	SIGNAL_LEFT
	call	READ_VOLTAGE_8
	subwf	voltage_right,f
	return

	;************************************************************************
	; W contient le code pour la base issu de la table			*
	; tablit la borne de base						*
	;************************************************************************

DETERMINE_SIGNAL_BASE:
	movwf	FSR
	movf	FSR,f
	movlw	TYPE_BASE
	btfsc	_Z
	goto	display_base_a_gauche
	btfss	FSR,0
	goto	display_base_a_droite
	movwf	pin_middle
	return

display_base_a_gauche:
	movwf	pin_left
	return

display_base_a_droite:
	movwf	pin_right
	return
	
	;************************************************************************
	; W contient le code pour la grille issu de la table			*
	; tablit la borne de base						*
	;************************************************************************

DETERMINE_SIGNAL_GRILLE:

	movwf	FSR
	movf	FSR,f
	movlw	TYPE_GRILLE
	btfsc	_Z
	goto	display_grille_a_gauche
	btfss	FSR,0
	goto	display_grille_a_droite
	movwf	pin_middle
	return

display_grille_a_gauche:

	movwf	pin_left
	return

display_grille_a_droite:

	movwf	pin_right
	return

	;************************************************************************
	; W contient la position de la base:					*
	; W=0 base  gauche ; W=1 base au milieu ; w=2 base  droite		*
	; retourne la position de l'emetteur et du collecteur dans W		*
	; C est positionn s'il y a une erreur (transistor invalide)		*
	;************************************************************************

DETERMINE_BROCHAGE_BIPOLAIRE:

	movwf	FSR
	movwf	position_base
	
	; tablissement des niveaux de polarisation suivant le type NPN/PNP
		
	movf	position_base,f
	btfss	_Z
	goto	dbp_a1

	; base  gauche

	CONFIG_BROCHAGE position_base, position_emetteur, position_collecteur
	call	CALCUL_BETA		; retourne C=0 si OK
	btfss	_C			; erreur: on teste la combinaison suivante
	return

	CONFIG_BROCHAGE position_base, position_collecteur, position_emetteur
	call	CALCUL_BETA
	return

dbp_a1:
	btfss	position_base,0
	goto	dbp_a2

	; base au milieu

	CONFIG_BROCHAGE position_collecteur, position_base, position_emetteur
	call	CALCUL_BETA
	btfss	_C
	return

	CONFIG_BROCHAGE position_emetteur, position_base, position_collecteur
	call	CALCUL_BETA
	btfss	_C
	return

dbp_a2:

	; base  droite

	CONFIG_BROCHAGE position_emetteur, position_collecteur, position_base
	call	CALCUL_BETA
	btfss	_C
	return

	CONFIG_BROCHAGE position_collecteur, position_emetteur, position_base
	call	CALCUL_BETA
	return

	;************************************************************************
	; W contient la position de la grille:					*
	; W=0  gauche ; W=1 au milieu ; w=2  droite				*
	; retourne la position de la source et du drain dans W			*
	; C est positionn s'il y a une erreur (transistor invalide)		*
	;************************************************************************

DETERMINE_BROCHAGE_FET:
	movwf	FSR
	movwf	position_base
	
	; tablissement des niveaux de polarisation suivant le type N/P
	
	movlw	TYPE_NJFET
	xorwf	transistor_type,W
	btfss	_Z
	goto	dbfet_p
	
	; transistor NJFET
	
	movlw	VOLTAGE_5V
	movwf	polarisation_collecteur
	movwf	polarisation_base
	movlw	VOLTAGE_0V
	movwf	polarisation_emetteur
	goto	dbfet_debut
dbfet_p:
	movlw	VOLTAGE_0V
	movwf	polarisation_collecteur
	movwf	polarisation_base
	movlw	VOLTAGE_5V
	movwf	polarisation_emetteur
dbfet_debut:
	movf	FSR,f
	btfsc	_Z
	goto	dbfet_grille_left
	btfss	FSR,0
	goto	dbp_grille_right
	
	; grille au milieu
	
	movf	resistance_gauche_droite,W
	movwf	resistance_serie
	movlw	SIGNAL_MIDDLE|RESISTANCE_100K
	call	SET_RESISTANCE		; 100K sur la grille
	movf	polarisation_base,W
	iorlw	SIGNAL_MIDDLE
	call	SET_VOLTAGE		; grille branche sur le 0V pour canal N 5V pour canal P
	
	movlw	SIGNAL_RIGHT
	movwf	collecteur_1
	movlw	SIGNAL_LEFT
	movwf	emetteur_1
	goto	dbfet_01

dbp_grille_right:
	
	; grille  droite
	
	movf	resistance_gauche_milieu,W
	movwf	resistance_serie
	movlw	SIGNAL_RIGHT|RESISTANCE_100K
	call	SET_RESISTANCE		; 100K sur la grille
	movf	polarisation_base,W
	iorlw	SIGNAL_RIGHT
	call	SET_VOLTAGE		; branche sur le 0V pour canal N
	
	movlw	SIGNAL_LEFT
	movwf	collecteur_1
	movlw	SIGNAL_MIDDLE
	movwf	emetteur_1
	goto	dbfet_01

dbfet_grille_left:
	
	; grille  gauche
	
	movf	resistance_milieu_droite,W
	movwf	resistance_serie
	movlw	SIGNAL_LEFT|RESISTANCE_100K
	call	SET_RESISTANCE		; 100K sur la grille
	movf	polarisation_base,W
	iorlw	SIGNAL_LEFT
	call	SET_VOLTAGE		; branche sur le 0V pour canal N
	
	movlw	SIGNAL_RIGHT
	movwf	collecteur_1
	movlw	SIGNAL_MIDDLE
	movwf	emetteur_1
	
dbfet_01:
	
	; 1er essai avec collecteur_1  5V  travers 100R
	
	movlw	RESISTANCE_100R
	iorwf	collecteur_1,W
	call	SET_RESISTANCE		; 100R sur le drain
	movf	polarisation_collecteur,W
	iorwf	collecteur_1,W
	call	SET_VOLTAGE		; branche sur le 5V
	
	; et position emetteur_1  la masse  travers 1K
	
	movlw	RESISTANCE_100R
	iorwf	emetteur_1,W
	call	SET_RESISTANCE		; source  la masse  travers 1K
	movf	polarisation_emetteur,W
	iorwf	emetteur_1,W
	call	SET_VOLTAGE
	
	movf	collecteur_1,W
	call	READ_VOLTAGE_10		; retourne FSR:W=tension drain-masse 10 bits
	movwf	voltage_drain_l
	movf	FSR,W
	movwf	voltage_drain_h
	
	movf	emetteur_1,W
	call	READ_VOLTAGE_10		; retourne FSR:W=tension source-masse 10 bits
	movwf	voltage_source_l
	movf	FSR,W
	movwf	voltage_source_h
	
dbfet_02:
	
	; calcul du RDSON
	
	movlw	TYPE_NJFET
	xorwf	transistor_type,W
	btfsc	_Z
	goto	dbfet_12			; transistor canal N
	
	; il faut permuter les tensions source/drain
	
	movf	voltage_drain_l,W		; permutation des tensions (8 bits low)
	movwf	FSR
	movf	voltage_source_l,W
	movwf	voltage_drain_l
	movf	FSR,W
	movwf	voltage_source_l
	;
	movf	voltage_drain_h,W		; permutation des tensions (8 bits high)
	movwf	FSR
	movf	voltage_source_h,W
	movwf	voltage_drain_h
	movf	FSR,W
	movwf	voltage_source_h
dbfet_12:
	
	; calcul de l'cart voltage_drain - voltage_source
	; rsultat plac dans AARGB0:AARGB1
	
	movf	voltage_source_l,W
	subwf	voltage_drain_l,W
	movwf	AARGB1
	movf	voltage_source_h,W
	btfss	_C
	incfsz	voltage_source_h,W
	subwf	voltage_drain_h,W
	movwf	AARGB0
dbfet_13:
	
	; multiplication de l'cart par la rsistance 100R + Rpic = 125 ohms
	; (16 bits x 8 bits)
	
	call	CALCUL_DRAIN_SOURCE
	movf	resistance_serie,W
	call	SUB_RESISTANCE_FROM_DATA
	return

CALCUL_DRAIN_SOURCE:

	movlw	R100R+RPICLOW
	movwf	BARGB0
	call	FXM1608U			; rsultat dans AARGB0:AARGB1:AARGB2
	movf	voltage_source_l,W
	movwf	BARGB1
	movf	voltage_source_h,W
	movwf	BARGB0
	call	FXD2315U
	movf	AARGB1,W
	movwf	data_h
	movf	AARGB2,W
	movwf	data_l
	return



INIT_TIMER1:
	
	; initialisation du timer 1 pour interrution temps rel
	
	bcf	T1CON,T1OSCEN		; oscillateur off
	bcf	T1CON,T1CKPS1
	bcf	T1CON,T1CKPS0		; prescaler 1:1
	bsf	T1CON,NOT_T1SYNC
	bcf	T1CON,TMR1CS		; internal clock
	movlw	LOW (0xFFFF-RECHARGE_TIMER1)
	movwf	TMR1L
	movlw	HIGH (0xFFFF-RECHARGE_TIMER1)
	movwf	TMR1H
	bsf	T1CON,TMR1ON		; timer 1 on
	BANK1
	bsf	PIE1,TMR1IE		; enable timer 1 interrupt
	BANK0
	bsf	INTCON,GIE
	bsf	INTCON,PEIE		; enable peripheral interrupts
	return

TEMPORISATION:
	
	; temporisation gnrale
	
	movwf	tempo1
loop_temporisation:
	movf	tempo1,f
	btfss	_Z
	goto	loop_temporisation
	return

BIPOLAR_TO_MOSFET:

	; change les noms des broches pour un mosfet
	; collecteur -> drain base -> grille emetteur -> source
	; entre: W = type bipolaire
	; sortie: W = type mosfet
	
	movwf	FSR
	movlw	TYPE_EMETTEUR
	xorwf	FSR,W
	btfsc	_Z
	retlw	TYPE_SOURCE

	movlw	TYPE_BASE
	xorwf	FSR,W
	btfsc	_Z
	retlw	TYPE_GRILLE

	movlw	TYPE_COLLECTEUR
	xorwf	FSR,W
	btfsc	_Z
	retlw	TYPE_DRAIN
	return

CALCUL_BETA:
	
	; calcul du gain en courant du transistor
	; On utilise le transistor en emetteur suiveur avec une resistance
	; de 100K sur la base et une rsistance d'metteur de 1K
		
	; tablissement des niveaux de polarisation suivant le type NPN/PNP
	
	movlw	TYPE_NPN
	xorwf	transistor_type,W
	btfss	_Z
	goto	beta_01
	
	; transistor NPN
	
	movlw	VOLTAGE_5V
	movwf	polarisation_collecteur
	movwf	polarisation_base
	movlw	VOLTAGE_0V
	goto	beta_02

beta_01:

	; transistor PNP

	movlw	VOLTAGE_0V
	movwf	polarisation_collecteur
	movwf	polarisation_base
	movlw	VOLTAGE_5V
	
beta_02:
	
	movwf	polarisation_emetteur
	movwf	polarisation_base_zero
	
	; le collecteur est connect au +5V (NPN) ou  la masse (PNP)
	
	movlw	RESISTANCE_100R
	iorwf	position_collecteur,W
	call	SET_RESISTANCE
	movf	polarisation_collecteur,W
	iorwf	position_collecteur,W
	call	SET_VOLTAGE
	;
	; l'metteur est connect  la masse (NPN) ou au +5V (PNP)  travers 1K
	;
	movlw	RESISTANCE_1K
	iorwf	position_emetteur,W
	call	SET_RESISTANCE
	movf	polarisation_emetteur,W
	iorwf	position_emetteur,W
	call	SET_VOLTAGE
	
	; la base est connecte tout d'abord au +5V (NPN) ou  la masse (PNP)  travers 100K
	
	movlw	RESISTANCE_100K
	iorwf	position_base,W
	call	SET_RESISTANCE
	movf	polarisation_base,W
	iorwf	position_base,W
	call	SET_VOLTAGE
	call	LCD_LONGDELAY		; tempo pour charge capa grille si mosfet
	
	; le courant de base est dtermin par la mesure de la tension de base
	; PNP: Ib = Vb/Rb
	; NPN: Ib = (5V-Vb)/Rb
	; Le courant collecteur est dtermin par la tension mesure sur l'metteur
	; PNP: Ie = (5V-Ve)/Re
	; NPN: Ie = Ve/Re
	; Beta = (Ve/Ib)*(Rb/Re) - 1
	
	movf	position_base,W
	call	READ_VOLTAGE_8		; un coup pour rien pour stabilisation
	call	DELAY_CONVERSION
	movf	position_base,W
	call	READ_VOLTAGE_8
	movwf	voltage_base
	comf	voltage_base,f		; ib = (5V-VB)/RB
	movf	voltage_base,W
	movwf	courant_base		; pour mosfet
	
	movf	position_emetteur,W
	call	READ_VOLTAGE_8		; un coup pour rien pour stabilisation
	call	DELAY_CONVERSION
	movf	position_emetteur,W
	call	READ_VOLTAGE_8
	movwf	voltage_emetteur
	movf	voltage_emetteur,f
	btfsc	_Z
	goto	beta_erreur
	incf	voltage_emetteur,f
	btfsc	_Z
	goto	beta_erreur
	decf	voltage_emetteur,f

	; on va refaire une mesure de la tension d'emetteur en dsactivant le signal de base
	; modif du 3/11/2004

	movf	polarisation_base_zero,W
	iorwf	position_base,W
	call	SET_VOLTAGE
	call	LCD_LONGDELAY
	movf	position_base,W
	call	READ_VOLTAGE_8

	subwf	courant_base,f		; ecart du courant base

	movf	position_emetteur,W
	call	READ_VOLTAGE_8		; un coup pour rien pour stabilisation
	call	DELAY_CONVERSION
	movf	position_emetteur,W
	call	READ_VOLTAGE_8
	movwf	voltage_emetteur_zero
	subwf	voltage_emetteur,f	; soustraction de la valeur zero
		
	; on va maintenant corriger les valeurs selon le type de transistor
	
	movlw	TYPE_NPN
	xorwf	transistor_type,W
	btfsc	_Z
	goto	beta_03			; transistor NPN

	; transistor PNP
	; si C=0 OK sinon pas un transistor valide
	; si 5V - voltage_emetteur_zero trop grand -> pas bon

	btfsc	_C
	goto	beta_erreur

	comf	voltage_emetteur_zero,f
	movlw	.255 - LIMITE_FUITE_COLLECTEUR	; limite pour courant de fuite C-E
	addwf	voltage_emetteur_zero,W
	btfsc	_C
	goto	beta_erreur

	movlw	.94			; Rb/ (Rc+Rpichigh)
	movwf	BARGB0
	comf	voltage_base,f
	comf	voltage_emetteur,f
	goto	beta_031

beta_03:
	
	; transistor NPN
	; si C=1 OK sinon pas un transistor valide, on dclare beta=0

	btfss	_C
	goto	beta_erreur

	movlw	.255 - LIMITE_FUITE_COLLECTEUR	; limite pour courant de fuite C-E
	addwf	voltage_emetteur_zero,W
	btfsc	_C
	goto	beta_erreur

	movlw	.97			; Rb/ (Rc+Rpiclow)
	movwf	BARGB0

beta_031:

	; si courant_base = 0..3 il peut s'agir d'un mosfet; en tout cas on ne calcule pas le beta

	movf	courant_base,W
	addlw	.252
	btfsc	_C
	goto	beta_032

	; pas de variation du courant de base (grille)
	; si la tension d'emetteur a vari c'est qu'il s'agit d'un mosfet

	movf	voltage_emetteur,w
	addlw	.245			; tension de seuil maximum = 4,5V
	btfss	_C
	goto	beta_erreur		; tension d'emetteur trop faible -> erreur

	; on a affaire  un mosfet; calcul de la tension de seuil

	comf	voltage_emetteur,f	; voltage_emetteur = 5V - Vth
	rlf	voltage_emetteur,W
	movwf	data_l
	clrf	data_h
	btfsc	_C
	incf	data_h,f

	; on va changer le type de transistor ici
	; NPN -> canal N
	; PNP -> canal P

	movlw	TYPE_NPN
	xorwf	transistor_type,W
	movlw	TYPE_NMOSFET
	btfss	_Z
	movlw	TYPE_PMOSFET
	movwf	transistor_type
	bcf	_C
	return
		
beta_032:

	; calcul final
	
	clrf	AARGB0
	movf	voltage_emetteur,W
	movwf	AARGB1
	call	FXM1608U
	movf	AARGB1,W
	movwf	AARGB0
	movf	AARGB2,W
	movwf	AARGB1

	movf	voltage_base,W
	movwf	BARGB0
	call	FXD1608U
	movf	AARGB0,W
	movwf	data_h
	movf	AARGB1,W
	movwf	data_l

	; il faut tester si le beta est dans les limites

	movf	data_h,W
	addlw	.252			; C=1 si data_h > 3
	btfsc	_C
	return				; beta > 1023 : trop grand

	movlw	0
	addwf	data_h,f		; C=0
	btfss	_Z
	return				; 1024 > beta > 256

	; beta < 256

	movf	data_l,W
	sublw	.20			; beta minimum
	return

beta_erreur:
	clrf	data_h
	clrf	data_l
	bsf	_C
	return

CLEAR_SIGNAL:
	
	; initialise les sorties  0V avec 100K
	
	CONFIG_VOLTAGE VOLTAGE_0V, VOLTAGE_0V, VOLTAGE_0V
	
	CONFIG_RESISTANCE RESISTANCE_100K, RESISTANCE_100K, RESISTANCE_100K
	
	return



TEST_FOR_JFET:
	
	; 2 niveaux proches de 0V et un niveau proche de 0,8V -> transistor JFETN
	; 2 niveaux proches de 0V et un niveau proche de 5V -> transistor JFETP
	;
	; la routine retourne le type de transistor dans le quartet faible de W
	; la position de la grille est retourne dans le quartet faible de W
	
	clrf	jfet_count_low
	clrf	jfet_count_high_n
	clrf	jfet_count_high_p
	
	; broche gauche
	
	movf	voltage_left,W
	movwf	FSR
	call	test_jfet_terminal		; retourne Z!=0 si pas d'erreur
	iorlw	0				; positionne Z
	btfsc	_Z
	retlw	TYPE_INCONNU
	bcf	terminal_levels,SIGNAL_LEFT
	movwf	FSR
	xorlw	1				; niveau bas ?
	btfss	_Z
	goto	tfj_01
	incf	jfet_count_low,f
	goto	tfj_02
tfj_01:
	bsf	terminal_levels,SIGNAL_LEFT	; niveau haut
	movf	FSR,W
	xorlw	2				; niveau haut de type P ?
	btfsc	_Z
	incf	jfet_count_high_p,f
	movf	FSR,W
	xorlw	3				; niveau haut de type N ?
	btfsc	_Z
	incf	jfet_count_high_n,f	

tfj_02:
	
	; broche du milieu
	
	movf	voltage_middle,W
	movwf	FSR
	call	test_jfet_terminal	; retourne C=0 si pas d'erreur
	iorlw	0
	btfsc	_Z
	retlw	TYPE_INCONNU
	bcf	terminal_levels,SIGNAL_MIDDLE
	movwf	FSR
	xorlw	1
	btfss	_Z
	goto	tfj_03
	incf	jfet_count_low,f
	goto	tfj_04
tfj_03:
	bsf	terminal_levels,SIGNAL_MIDDLE
	movf	FSR,W
	xorlw	2				; niveau haut de type P ?
	btfsc	_Z
	incf	jfet_count_high_p,f
	movf	FSR,W
	xorlw	3				; niveau haut de type N ?
	btfsc	_Z
	incf	jfet_count_high_n,f

tfj_04:
	
	; broche de droite
	
	movf	voltage_right,W
	movwf	FSR
	call	test_jfet_terminal	; retourne C=0 si pas d'erreur
	movwf	FSR
	iorlw	0
	btfsc	_Z
	retlw	TYPE_INCONNU
	bcf	terminal_levels,SIGNAL_RIGHT
	movwf	FSR
	xorlw	1
	btfss	_Z
	goto	tfj_05
	incf	jfet_count_low,f
	goto	tfj_06
tfj_05:
	bsf	terminal_levels,SIGNAL_RIGHT
	movf	FSR,W
	xorlw	2				; niveau haut de type P ?
	btfsc	_Z
	incf	jfet_count_high_p,f
	movf	FSR,W
	xorlw	3				; niveau haut de type N ?
	btfsc	_Z
	incf	jfet_count_high_n,f
	bsf	terminal_levels,SIGNAL_RIGHT
tfj_06:
	;
	; les flags de terminal_levels sont positionns
	;
	; jfet_count_low doit valoir 2
	; jfet_count_high_n doit valoir 1 ou 0
	; jfet_count_high_p doit valoir 0 ou 1
	
	movlw	2
	xorwf	jfet_count_low,W
	btfss	_Z
	retlw	TYPE_INCONNU
	decf	jfet_count_high_p,W
	btfsc	_Z
	goto	tfj_07			; jfet de type P
	decf	jfet_count_high_n,W
	btfss	_Z
	retlw	TYPE_INCONNU
	
	; jfet de type N
	
	movlw	1
	xorwf	terminal_levels,W
	btfsc	_Z
	retlw	TYPE_NJFET|BASE_LEFT
	movlw	2
	xorwf	terminal_levels,W
	btfsc	_Z
	retlw	TYPE_NJFET|BASE_MIDDLE
	retlw	TYPE_NJFET|BASE_RIGHT
tfj_07:
	
	; jfet de type P
	
	movlw	1
	xorwf	terminal_levels,W
	btfsc	_Z
	retlw	TYPE_PJFET|BASE_LEFT
	movlw	2
	xorwf	terminal_levels,W
	btfsc	_Z
	retlw	TYPE_PJFET|BASE_MIDDLE
	retlw	TYPE_PJFET|BASE_RIGHT

test_jfet_terminal:
	;
	; entre: FSR contient le voltage
	; sortie: W=0 : erreur voltage
	;	  W=1 niveau bas
	;	  W=2 si niveau haut de type 5V (pour canal P)
	;	  W=3 si niveau haut de type 0,7V (pour canal N)
	
	movlw	MAXIMUM_LOW_LEVEL_JFET
	subwf	FSR,W
	btfss	_C				; skip si au-dessus du niveau bas
	goto	test_jfet_minimum_level		; en-dessous du niveau bas maximum -> test pour minimum de tension
	
	; au-dessus du niveau bas - il faut tester si le niveau haut est de type 5V ou de type 0,8V
	
	movlw	MINIMUM_HIGH_LEVEL_PJFET
	subwf	FSR,W
	btfsc	_C				; skip si au-dessus du niveau haut
	retlw	2
	
	; on est en-dessous du niveau haut minimal du type 5V
	; on va tester si on est au-dessus du niveau haut minimum de type 0,7V
	
	movlw	MINIMUM_HIGH_LEVEL_NJFET
	subwf	FSR,W
	btfss	_C
	retlw	0				; en-dessous du niveau haut minimum -> erreur
	
	; on est au-dessus du niveau haut minimum
	; on va tester si on est au-dessous du niveau haut maximum de type 0,7V
	
	movlw	MAXIMUM_HIGH_LEVEL_NJFET
	subwf	FSR,W
	btfss	_C
	retlw	3
	retlw	0

test_jfet_minimum_level:
	
	; on est en-dessous du niveau bas maximum
	
	movlw	MINIMUM_LOW_LEVEL_JFET
	subwf	FSR,W
	btfss	_C				; skip si au-dessus du minimum
	retlw	0				; pas un jfet
	retlw	1

erreur_voltage_jfet:
	
	; tension incorrecte pour un FET sur la broche du transistor
	
	retlw	0


TEST_CALIBRATION:
	;
	; teste le mode calibration
	; retourne W=1 si calibration
	; W=0 sinon
	; Le mode calibration est obtenu en forant au +5V l'entre droite (AN0)
	;

	CONFIG_RESISTANCE RESISTANCE_5K, RESISTANCE_5K, RESISTANCE_5K

	CONFIG_VOLTAGE VOLTAGE_0V, VOLTAGE_0V, VOLTAGE_0V

	; lecture de la tension
	; tension = 0V -> pas de calibration
	; tension = 5V -> calibration
	
	movlw	SIGNAL_RIGHT
	call	READ_VOLTAGE_8
	sublw	.10
	btfss	_C
	retlw	1				; retour pour mode calibration
	retlw	0				; retour pour mode normal

MESURE_VOLTAGE_X:
	;
	; on va mesurer avec prcision la tension entre la borne X et la masse
	; on va chercher 2 mesures conscutives identiques
	; entre: W = SIGNAL_LEFT ou SIGNAL_RIGHT ou SIGNAL_MIDDLE
	; sortie: data_h:data_l
	;
	movwf	temporaire
loop_stab_mesure_cc1:
	movf	temporaire,W		; SIGNAL_LEFT
	call	READ_VOLTAGE_8
	sublw	.240			; seuil  dfinir
	btfss	_C
	goto	loop_stab_mesure_cc1
	
	movf	temporaire,W		; SIGNAL_LEFT
	call	READ_VOLTAGE_10		; retourne FSR:W
	movwf	BARGB1
	movf	FSR,W
	movwf	BARGB0			; voltage_drain (h-l) 1re mesure drain-masse
	
	movf	temporaire,W		; SIGNAL_LEFT
	call	READ_VOLTAGE_10		; retourne FSR:W
	movwf	AARGB1
	movf	FSR,W
	movwf	AARGB0			; AARGB0:AARGB1 2ime mesure drain-masse
	
	movf	BARGB1,W
	xorwf	AARGB1,W
	btfss	_Z			; skip si mmes valeurs
	goto	loop_stab_mesure_cc1	; valeurs low diffrentes : on recommence

	movf	BARGB0,W
	xorwf	AARGB0,W
	btfss	_Z
	goto	loop_stab_mesure_cc1	; valeurs high diffrentes: on recommence
	return

CALIBRATION:
	;
	; Routine pour la calibration des rsistances des commutateurs analogiques
	; L'utilisateur doit court-circuiter les bornes du connecteur avant de retirer le cavalier
	; La routine mmorise les rsistances des trois commutateurs par paires
	; (droite+milieu, droite+gauche, gauche+droite)
	;
	call	LCD_CLRSCR
	movlw	CODE_MESSAGE_CAL
	call	DISPLAY_MESSAGE
	
	; on doit ter le cavalier pour lancer la calibration
	
cal_loop_strap:
	call	TEST_CALIBRATION	; retourne W=0 en mode normal W=1 en mode calibration
	addlw	0			; Z=1: strap retir
	btfss	_Z
	goto	cal_loop_strap
	call	LCD_CLRSCR

	CONFIG_RESISTANCE 	RESISTANCE_100R, RESISTANCE_100R, RESISTANCE_100K
	CONFIG_VOLTAGE		VOLTAGE_5V, VOLTAGE_0V, VOLTAGE_0V	

	; mesure rsistance gauche-milieu

	movlw	SIGNAL_LEFT
	call	MESURE_VOLTAGE_X		; mesure de la tension drain-masse  : voltage_drain (_h, _l)
	movf	data_l,W
	movwf	voltage_drain_l
	movf	data_h,W
	movwf	voltage_drain_h

	movlw	SIGNAL_MIDDLE
	call	MESURE_VOLTAGE_X		; mesure de la tension source-masse : voltage_source (_h,_l)
	movf	data_l,W
	movwf	voltage_source_l
	movf	data_h,W
	movwf	voltage_source_h

	call	CALCUL_RESISTANCE_SERIE
	movf	data_l,W
	movwf	resistance_gauche_milieu
	call	BINAIRE_TO_DECIMAL
	clrf	FSR
	bsf	FSR, R_G_M
	call	DISPLAY_RESISTANCE
	movlw	DUREE_AFF_RESISTANCE
	call	TEMPORISATION

	; mesure rsistance gauche-droite

	CONFIG_RESISTANCE 	RESISTANCE_100R, RESISTANCE_100K, RESISTANCE_100R
	CONFIG_VOLTAGE		VOLTAGE_5V, VOLTAGE_0V, VOLTAGE_0V	

	movlw	SIGNAL_LEFT
	call	MESURE_VOLTAGE_X		; mesure de la tension drain-masse  : voltage_drain (_h, _l)
	movf	data_l,W
	movwf	voltage_drain_l
	movf	data_h,W
	movwf	voltage_drain_h

	movlw	SIGNAL_RIGHT
	call	MESURE_VOLTAGE_X		; mesure de la tension source-masse : voltage_source (_h,_l)
	movf	data_l,W
	movwf	voltage_source_l
	movf	data_h,W
	movwf	voltage_source_h

	call	CALCUL_RESISTANCE_SERIE
	movf	data_l,W
	movwf	resistance_gauche_droite
	call	BINAIRE_TO_DECIMAL
	clrf	FSR
	bsf	FSR, R_G_D
	call	DISPLAY_RESISTANCE
	movlw	DUREE_AFF_RESISTANCE
	call	TEMPORISATION
	
	; mesure rsistance milieu-droite

	CONFIG_RESISTANCE 	RESISTANCE_100K, RESISTANCE_100R, RESISTANCE_100R
	CONFIG_VOLTAGE		VOLTAGE_0V, VOLTAGE_5V, VOLTAGE_0V	

	movlw	SIGNAL_MIDDLE
	call	MESURE_VOLTAGE_X		; mesure de la tension drain-masse  : voltage_drain (_h, _l)
	movf	data_l,W
	movwf	voltage_drain_l
	movf	data_h,W
	movwf	voltage_drain_h

	movlw	SIGNAL_RIGHT
	call	MESURE_VOLTAGE_X		; mesure de la tension source-masse : voltage_source (_h,_l)
	movf	data_l,W
	movwf	voltage_source_l
	movf	data_h,W
	movwf	voltage_source_h

	call	CALCUL_RESISTANCE_SERIE
	movf	data_l,W
	movwf	resistance_milieu_droite
	call	BINAIRE_TO_DECIMAL
	clrf	FSR
	bsf	FSR, R_M_D
	call	DISPLAY_RESISTANCE
	movlw	DUREE_AFF_RESISTANCE
	call	TEMPORISATION
	
	; mmorisation des rsistances en eeprom

	movf	resistance_gauche_milieu,W
	movwf	FSR			; valeur  crire
	movlw	R_G_M			; adresse de l'octet  crire
	call	ECRITURE_EEPROM

	movf	resistance_gauche_droite,W
	movwf	FSR			; valeur  crire
	movlw	R_G_D			; adresse de l'octet  crire
	call	ECRITURE_EEPROM

	movf	resistance_milieu_droite,W
	movwf	FSR			; valeur  crire
	movlw	R_M_D			; adresse de l'octet  crire
	call	ECRITURE_EEPROM
	
	; test si bonne criture; s'il reste de la place

	;movlw	R_G_M
	;call	LECTURE_EEPROM
	;xorwf	resistance_gauche_milieu,w
	;btfss	_Z
	;goto	erreur_eeprom

	;movlw	R_G_D
	;call	LECTURE_EEPROM
	;xorwf	resistance_gauche_droite,w
	;btfss	_Z
	;goto	erreur_eeprom

	;movlw	R_M_D
	;call	LECTURE_EEPROM
	;xorwf	resistance_milieu_droite,w
	;btfss	_Z
	;goto	erreur_eeprom

	; fin de calibration
	; on crit un code pour indiquer la calibration

	movlw	CODE_EEPROM
	movwf	FSR			; valeur  crire
	movlw	ADR_CODE_EEPROM		; adresse de l'octet  crire
	call	ECRITURE_EEPROM
	return

;erreur_eeprom:
;	goto	erreur_eeprom


LECTURE_RESISTANCES:
	
	; lecture des valeurs des rsistances dans l'eeprom
	
	movlw	R_G_M
	call	LECTURE_EEPROM
	movwf	resistance_gauche_milieu

	movlw	R_G_D
	call	LECTURE_EEPROM
	movwf	resistance_gauche_droite
	
	movlw	R_M_D
	call	LECTURE_EEPROM
	movwf	resistance_milieu_droite

	return

TEST_CAL_DONE:
	;
	; teste si la calibration a t effectue
	; retourne Z=1 si calibration effectue
	;          Z=0 sinon

	movlw	ADR_CODE_EEPROM
	call	LECTURE_EEPROM
	xorlw	CODE_EEPROM
	return

ECRITURE_EEPROM:
	;
	; effectue l'criture d'un octet dans l'eepom du pic
	; entre: FSR = data  crire
	;         W = adresse d'criture

	BANK2

	movwf	EEADR
	movf	FSR,W
	movwf	EEDATA
	
	BANK3

	bcf	EECON1, EEPGD		; access to data memory
	bsf	EECON1, WREN		; enable write
	bcf	INTCON, GIE		; disable all interrupts
	movlw	0x55
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1, WR
	bsf	INTCON, GIE
	bcf	EECON1, WREN
loop_write_eeprom:
	btfsc	EECON1, WR
	goto	loop_write_eeprom
	BANK0

	return

LECTURE_EEPROM:
	;
	; Effectue une lecture dans l'eeprom
	; Entre: W = adresse
	; Sortie: W

	BANK2
	movwf	EEADR

	BANK3
	bcf	EECON1, EEPGD
	bsf	EECON1, RD
	
	BANK2
	movf	EEDATA,W

	BANK0
	return


CALCUL_RESISTANCE_SERIE:
	;
	; Effectue le calcul Rserie = (R100R+RPIC)*Vdrain/Vsource - (R100R+RPIC)
	; Entre: voltage_drain, voltage_source
	; Sortie: rsultat dans data_h:data_l

	; multiplication Voltage_drain par R100R + Rpic = 125 ohms
	; (16 bits x 8 bits)
	
	movf	voltage_drain_l,W
	movwf	AARGB1
	movf	voltage_drain_h,W
	movwf	AARGB0

	call	CALCUL_DRAIN_SOURCE
	movlw	R100R+RPICLOW
	call	SUB_RESISTANCE_FROM_DATA
	movlw	R100R+RPICLOW
	return

SUB_RESISTANCE_FROM_DATA:

	subwf	data_l,f
	btfsc	_C			; C=1 si pas de retenue
	goto	sub_rfd_01
	movf	data_h,f
	btfsc	_Z
	goto	sub_rfd_02
	decf	data_h,f
	goto	sub_rfd_01
sub_rfd_02:
	clrf	data_l
sub_rfd_01:	
	return

;	INCLUDE "display.inc"
;	INCLUDE "hard.inc"
;	INCLUDE "math.inc"
;	INCLUDE "fxd68.inc"

	; tables et messages - ne doivent pas chevaucher des pages de 255

DEBUT_MESSAGES:				; identificateur pour PCLATH

MESSAGE_INVITE:
	addwf	PCL,f
	DT	"T4001 V1.0",0

MESSAGE_CAL:
	addwf	PCL,f
	DT	"CALIBRATION",0

MESSAGE_ERRCAL:
	addwf	PCL,f
	DT	"ERREUR CAL",0

	ifdef LCD16X2

TABLE_TYPE_TRANSISTOR:
	addwf	PCL,f
	DT	"BIP-NPN "	; 1 NPN
	DT	"BIP-PNP "	; 2 PNP
	DT	"?       "	; 3 inconnu
	DT	"N-JFET  "	; 4 jfet N
	DT	"P-JFET  "	; 5 jfet p
	DT	"N-MOSFET"	; 6 mosfet N
	DT	"P-MOSFET"	; 7 mosfet P

	else

TABLE_TYPE_TRANSISTOR:
	addwf	PCL,f
	DT	"NPN "		; 1 NPN
	DT	"PNP "		; 2 PNP
	DT	"?   "		; 3 inconnu
	DT	"NJF "		; 4 jfet N
	DT	"PJF "		; 5 jfet p
	DT	"NMF "		; 6 mosfet N
	DT	"PMF "		; 7 mosfet P
	endif

TABLE_BIPOLAIRE:
	addwf	PCL,f
	retlw	TYPE_INCONNU				; 000 (tout en court-circuit)
	retlw	TYPE_PNP|BASE_LEFT			; 001 (base  gauche)
	retlw	TYPE_PNP|BASE_MIDDLE			; 010 (base au milieu)
	retlw	TYPE_NPN|BASE_RIGHT			; 011 (base  droite)
	retlw	TYPE_PNP|BASE_RIGHT			; 100 (base  droite)
	retlw	TYPE_NPN|BASE_MIDDLE			; 101 (base au milieu)
	retlw	TYPE_NPN|BASE_LEFT			; 110 (base  gauche)
	retlw	TYPE_INCONNU				; 111 (pas de transistor)
	
	INCLUDE "display.inc"
	INCLUDE "hard.inc"
	INCLUDE "math.inc"	
	INCLUDE "fxd68.inc"

	END


